/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

/**
 * {@link VerilogStandard} contains Verilog standards' identifiers.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public enum VerilogStandard {
  IEEE_1364_1995("1364-1995", 0),
  IEEE_1364_2001_NC("1364-2001-noconfig", 1),
  IEEE_1364_2001("1364-2001", 2),
  IEEE_1364_2005("1364-2005", 3);

  private final String name;
  private final int level;

  private VerilogStandard(final String name, final int level) {
    this.name= name;
    this.level = level;
  }

  public String getName() {
    return name;
  }

  public int getLevel() {
    return level;
  }

  public boolean isCompliantWith(final VerilogStandard standard) {
    return level <= standard.level;
  }
}

