/*
 * Copyright 2013-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.VerilogNode;

import java.util.ArrayList;
import java.util.List;

/**
 * {@link VerilogSyntaxBackends} implements a composite AST-level back-end.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogSyntaxBackends extends VerilogSyntaxBackend {
  /** Available back-ends. */
  private List<VerilogSyntaxBackend> backends = new ArrayList<>();

  public VerilogSyntaxBackends() {
    super("all-syntax-backends");
  }

  /**
   * Registers the back-end.
   * 
   * @param backend the back-end to be registered.
   */
  public void add(final VerilogSyntaxBackend backend) {
    InvariantChecks.checkNotNull(backend);
    backends.add(backend);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void start(final VerilogNode root) {
    InvariantChecks.checkNotNull(root);

    for (final VerilogSyntaxBackend backend : backends) {
      VerilogLogger.debug("Starting the backend '%s'...", backend.getName());
      backend.start(root);
    }
  }
}
