/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogPath;

import java.util.EnumSet;

/**
 * {@link VerilogDisableStatement} represents disable statements.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogDisableStatement extends VerilogStatement {
  public static final Tag TAG = Tag.DISABLE_STATEMENT;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.noneOf(Tag.class);

  /** Path to the object being disabled. */
  private VerilogPath path;

  /**
   * Creates a disable statement.
   * 
   * @param parent the parent node.
   */
  public VerilogDisableStatement(final VerilogNode parent) {
    super(TAG, parent);
  }

  /**
   * Creates a copy of the disable statement.
   * 
   * @param other the disable statement to be copied.
   * @param parent the parent node.
   */
  public VerilogDisableStatement(final VerilogDisableStatement other, final VerilogNode parent) {
    super(other, parent);

    this.path = other.path != null ? other.path.clone() : null;
  }

  /**
   * Returns the path to the object being disabled.
   * 
   * @return the path to the object.
   */
  public VerilogPath getPath() {
    return path;
  }

  /**
   * Sets the path to the object being disabled.
   * 
   * @param path the path to be set.
   */
  public void setPath(final VerilogPath path) {
    InvariantChecks.checkNotNull(path);
    this.path = path;
  }

  @Override
  public VerilogDisableStatement clone() {
    return new VerilogDisableStatement(this, getParentNode());
  }
}
