/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import java.util.EnumSet;

/**
 * {@link VerilogGenerate} represents the abstract syntax of the generate construct.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogGenerate extends VerilogNode {
  public static final Tag TAG = Tag.GENERATE;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.of(
      Tag.DECLARATION,
      Tag.DEFINE_PARAMETER,
      Tag.PROCEDURE,
      Tag.INSTANTIATION,
      Tag.ACTIVITY,
      Tag.ASSIGN,
      Tag.IF_GENERATE,
      Tag.CASE_GENERATE,
      Tag.LOOP_GENERATE
  );

  /**
   * Creates a generate construct.
   * 
   * @param parent the parent node.
   */
  public VerilogGenerate(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.PSEUDO_BLOCK, parent);
  }

  /**
   * Creates a copy of the generate construct.
   * 
   * @param other the generate construct to be copied.
   * @param parent the parent node.
   */
  public VerilogGenerate(final VerilogGenerate other, final VerilogNode parent) {
    super(other, parent);
  }

  @Override
  public VerilogGenerate clone() {
    return new VerilogGenerate(this, getParentNode());
  }
}
