/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

import java.util.EnumSet;

/**
 * {@link VerilogWaitStatement} represents the abstract syntax of the wait statement.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogWaitStatement extends VerilogStatement {
  public static final Tag TAG = Tag.WAIT_STATEMENT;
  public static final EnumSet<Tag> TAGS_CHILDREN = VerilogStatement.TAGS;

  /** Wait expression. */
  private VerilogExpression expression;

  /**
   * Creates a wait statement.
   * 
   * @param parent the parent node.
   */
  public VerilogWaitStatement(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.ELEMENT, parent);
  }

  /**
   * Creates a copy of the wait statement.
   * 
   * @param other the wait statement to be copied.
   * @param parent the parent node.
   */
  public VerilogWaitStatement(final VerilogWaitStatement other, final VerilogNode parent) {
    super(other, parent);

    this.expression = other.expression != null ? other.expression.clone() : null;
  }

  /**
   * Returns the wait expression.
   * 
   * @return the expression.
   */
  public VerilogExpression getExpression() {
    return expression;
  }

  /**
   * Sets a wait expression.
   * 
   * @param expression the expression to be set.
   */
  public void setExpression(final VerilogExpression expression) {
    InvariantChecks.checkNotNull(expression);
    this.expression = expression;
  }

  /**
   * Returns the statement.
   * 
   * @return the statement.
   */
  public VerilogStatement getStatement() {
    return (VerilogStatement) items(TAGS_CHILDREN).iterator().next();
  }

  /**
   * Sets the statement.
   * 
   * @param statement the statement to be set.
   */
  public void setStatement(final VerilogStatement statement) {
    add(statement);
  }

  @Override
  public VerilogWaitStatement clone() {
    return new VerilogWaitStatement(this, getParentNode());
  }
}
