/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.transformer;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.util.InvariantChecks;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

/**
 * {@link VerilogTransformerComposite} implements a composite transformer.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogTransformerComposite extends VerilogTransformer {
  /** Transformers to be applied to an expression. */
  private final List<VerilogTransformer> transformers = new ArrayList<>();

  public VerilogTransformerComposite() {}

  public VerilogTransformerComposite(final VerilogTransformer ... transformers) {
    addAll(Arrays.asList(transformers));
  }

  public void add(final VerilogTransformer transformer) {
    InvariantChecks.checkNotNull(transformer);
    this.transformers.add(transformer);
  }

  public void addAll(final Collection<VerilogTransformer> transformers) {
    InvariantChecks.checkNotNull(transformers);
    this.transformers.addAll(transformers);
  }

  
  
  @Override
  public Node transform(final Node node) {
    Node result = node;

    for (final VerilogTransformer transformer : transformers) {
      result = transformer.transform(result);
    }

    return result;
  }
}
