
`timescale 1ns/100ps

// модель двухпортовой памяти

module mem_2p (
DO,
RD_A,
WR_A,
DI,
RD_CLK,
WR_CLK,
CE_N_RD,
CE_N_WR,
OE_N
);

//------Parameters declaration:---------
	parameter  ADDR_SIZE   = 4;
	parameter  DATA_SIZE   = 8;
//--------------------------------------

output [(DATA_SIZE-1):0] DO;      // Data Output

input  [(ADDR_SIZE-1):0] RD_A;        //Read Address
input  [(ADDR_SIZE-1):0] WR_A;        //Write Address
input  [(DATA_SIZE-1):0] DI;      // Data Input
input  RD_CLK;            // Read Clock
input  WR_CLK;            // Write Clock
input  CE_N_RD;            // Read Chip Select Enable   --- Active Low
input  CE_N_WR;            // Write Chip Select Enable   --- Active Low
input  OE_N;            // Output Enable --- Active Low


reg   [(DATA_SIZE-1):0] DataIn_R_WR;
reg   [(ADDR_SIZE-1):0] Address_R_RD;
reg   [(ADDR_SIZE-1):0] Address_R_WR;
reg   Clock_RD;
reg   Clock_WR;
reg   CE_N_R_RD;
reg   CE_N_R_WR;
reg   [(DATA_SIZE-1):0] memArray [((1<<ADDR_SIZE)-1):0];
reg   [(DATA_SIZE-1):0] Dout_TR_RD;
integer  j;
integer  k;
reg   coll_sig;
reg   read_flag;
reg   inWrite;
reg   inRead;




initial begin
 // Initializing Memory Array to x
 	for (k=0; k<(1<<ADDR_SIZE); k=k+1) begin
        memArray[k] = {(1<<DATA_SIZE){1'bx}};
	end
end


always @(posedge RD_CLK) begin
  inRead <= 1'b1;
  read_flag <= 1'b1;
  CE_N_R_RD <= CE_N_RD;
end

always @(posedge WR_CLK) begin
  inWrite <= 1'b1;
  CE_N_R_WR <= CE_N_WR;
end


always @(RD_CLK) begin
    if (!CE_N_RD || CE_N_RD === 1'bx)
        Clock_RD <= RD_CLK;
    else
        Clock_RD <= 1'b0;
end

always @(WR_CLK) begin
     if (!CE_N_WR || CE_N_WR === 1'bx)
        Clock_WR <= WR_CLK;
     else
        Clock_WR <= 1'b0;
end

always @(posedge RD_CLK) begin
  Address_R_RD <=  RD_A;

  if(!CE_N_RD) begin
    if(RD_A >= (1<<ADDR_SIZE))
      $display("%m %t Read Address:%h  is out of bounds",$time,RD_A);
  end

end

always @(posedge WR_CLK) begin
  Address_R_WR <=  WR_A;

  if(!CE_N_WR) begin
    if(WR_A >= (1<<ADDR_SIZE))
      $display("%m %t Write Address:%h  is out of bounds",$time,WR_A);
  end

  DataIn_R_WR  <=  DI;

end

always @( posedge Clock_WR) begin

    if(^Address_R_WR === 1'bx || CE_N_R_WR === 1'bx) begin
        if(!CE_N_R_WR || CE_N_R_WR === 1'bx) begin
            if(^Address_R_WR === 1'bx) begin
                for(j=0; j<(1<<ADDR_SIZE); j=j+1) begin
                    $display("%m %t All Address go to X",$time);
                    memArray[j] = {(1<<DATA_SIZE){1'bx}};
                end
            end else begin
                if(CE_N_R_WR === 1'bx) begin
                    $display("%m %t Write Address:%h go to X",$time,Address_R_WR);
                    memArray[Address_R_WR] = {(1<<DATA_SIZE){1'bx}};
                end
            end
        end
    end else begin
        if(!CE_N_R_WR) begin
            memArray[Address_R_WR] = DataIn_R_WR;
        end
    end

end

always @(posedge Clock_WR or posedge Clock_RD or posedge inRead or posedge inWrite) begin
    coll_sig = inWrite & inRead;
    if((!CE_N_R_RD && (^Address_R_RD === 1'bx)) || (CE_N_R_RD === 1'bx)) begin
        $display("%m %t Read X Address",$time);
        Dout_TR_RD = {(1<<DATA_SIZE){1'bx}};
    end else if(!CE_N_R_RD && (!CE_N_R_WR || (CE_N_R_WR === 1'bx)) && (((Address_R_WR == Address_R_RD) && coll_sig) || (^Address_R_WR === 1'bx))) begin
        $display("%m %t Collision detected",$time);
        Dout_TR_RD = {(1<<DATA_SIZE){1'bx}};
    end else if(!CE_N_R_RD && (CE_N_R_WR || !((Address_R_WR == Address_R_RD) && coll_sig)) && read_flag)  begin
        Dout_TR_RD = memArray[Address_R_RD];
    end

    read_flag = 1'b0;
end

assign DO = OE_N ? {(1<<DATA_SIZE){1'bx}} : Dout_TR_RD;


endmodule

